@@ -2,19 +2,17 @@ package DBIx::DataModel::Meta::Schema;
 use strict;
 use warnings;
 use parent 'DBIx::DataModel::Meta';
-use DBIx::DataModel::Meta::Utils;
+use DBIx::DataModel;
+use DBIx::DataModel::Meta::Utils qw/define_class define_readonly_accessors/;
 use DBIx::DataModel::Source::Join;
 use DBIx::DataModel::Meta::Source::Join;
 
-use Params::Validate     qw/validate SCALAR ARRAYREF CODEREF UNDEF BOOLEAN
-                                     OBJECT HASHREF/;
-use List::MoreUtils      qw/any lastval/;
-use Scalar::Util         qw/reftype/;
+use Params::Validate     qw/validate_with SCALAR ARRAYREF CODEREF UNDEF BOOLEAN
+                                          OBJECT HASHREF/;
+use List::MoreUtils      qw/any firstval lastval uniq/;
 use Module::Load         qw/load/;
-use Carp;
-use namespace::autoclean;
-
-{no strict 'refs'; *CARP_NOT = \@DBIx::DataModel::CARP_NOT;}
+use Carp::Clan           qw[^(DBIx::DataModel::|SQL::Abstract)];
+use namespace::clean;
 
 #----------------------------------------------------------------------
 # Params::Validate specification for new()
@@ -33,8 +31,6 @@ my $spec = {
   auto_update_columns          => {type => HASHREF, default => {}},
   no_update_columns            => {type => HASHREF, default => {}},
 
-
-
   # beware: more members of %$spec are added below
 };
 
@@ -43,25 +39,31 @@ for my $member (qw/table join/) {
   my $capitalized = ucfirst $member;
   my $parent     = "DBIx::DataModel::Source::$capitalized";
   my $meta_class = "DBIx::DataModel::Meta::Source::$capitalized";
-  $spec->{$member."_parent"} 
-    = {type    => SCALAR|ARRAYREF, default => $parent};
-  $spec->{$member."_metaclass"} 
-    = {type => SCALAR, isa => $meta_class, default => $meta_class};
+  $spec->{$member."_parent"}    = {type    => SCALAR|ARRAYREF,
+                                   default => $parent};
+  $spec->{$member."_metaclass"} = {type    => SCALAR, 
+                                   isa     => $meta_class,
+                                   default => $meta_class};
 }
 
 # parameters for optional subclasses of the builtin metaclasses
 for my $member (qw/association path type/) {
   my $capitalized = ucfirst $member;
   my $meta_class = "DBIx::DataModel::Meta::$capitalized";
-  $spec->{$member."_metaclass"}
-    = {type => SCALAR, isa => $meta_class, default => $meta_class};
+  $spec->{$member."_metaclass"} = {type    => SCALAR, 
+                                   isa     => $meta_class, 
+                                   default => $meta_class};
 }
 
-# parameters for optional subclass of the builtin statement class
+# parameters for optional subclasses of builtin classes
 my $statement_class = 'DBIx::DataModel::Statement';
-$spec->{statement_class} 
-  = {type => SCALAR, isa => $statement_class, default => $statement_class};
-
+$spec->{statement_class}        = {type    => SCALAR, 
+                                   isa     => $statement_class,
+                                   default => $statement_class};
+my $connected_source_class = 'DBIx::DataModel::ConnectedSource';
+$spec->{connected_source_class} = {type    => SCALAR, 
+                                   isa     => $connected_source_class,
+                                   default => $connected_source_class};
 
 
 #----------------------------------------------------------------------
@@ -72,7 +74,11 @@ sub new {
   my $class = shift;
 
   # check parameters
-  my $self = validate(@_, $spec);
+  my $self = validate_with(
+    params      => \@_,
+    spec        => $spec,
+    allow_extra => 0,
+   );
 
   # canonical representations (arrayref) for some attributes
   for my $attr (qw/isa table_parent parent join_parent/) {
@@ -90,7 +96,7 @@ sub new {
   bless $self, $class;
 
   # create the Perl class
-  DBIx::DataModel::Meta::Utils->define_class(
+  define_class(
     name    => $self->{class},
     isa     => $isa,
     metadm  => $self,
@@ -100,9 +106,7 @@ sub new {
 }
 
 # accessors for args passed to new()
-DBIx::DataModel::Meta::Utils->define_readonly_accessors(
-  __PACKAGE__, grep {$_ ne 'isa'} keys %$spec
- );
+define_readonly_accessors(__PACKAGE__, grep {$_ ne 'isa'} keys %$spec);
 
 # accessors for internal lists of other meta-objects
 foreach my $kind (qw/table association type join/) {
@@ -110,7 +114,7 @@ foreach my $kind (qw/table association type join/) {
   # retrieve list of meta-objects
   *{$kind."s"} = sub {
     my $self = shift;
-    return values %{$_[0]->{$kind}};
+    return values %{$self->{$kind}};
   };
 
   # retrieve single named object
@@ -123,6 +127,11 @@ foreach my $kind (qw/table association type join/) {
 }
 
 
+sub db_table {
+  my ($self, $db_name) = @_;
+  return firstval {uc($_->db_from) eq uc($db_name)} $self->tables;
+}
+
 
 #----------------------------------------------------------------------
 # PUBLIC FRONT-END METHODS FOR DECLARING SCHEMA MEMBERS
@@ -138,6 +147,9 @@ sub Table {
 
   # parse positional parameters (old syntax)
   my ($class_name, $db_name, @primary_key) = @_;
+  $db_name && @primary_key
+    or croak "not enough args to \$schema->Table(); "
+           . "did you mean \$schema->table() ?";
   $args{class}       ||= $class_name;
   $args{db_name}     ||= $db_name;
   $args{primary_key} ||= \@primary_key;
@@ -266,7 +278,7 @@ sub define_join {
   my %args = (
     schema         => $self,
     class          => $class_name,
-    parents        => [map {$_->{table}} @$joins],
+    parents        => [uniq map {$_->{table}} @$joins],
     sqla_join_args => \@sqla_join_args,
     aliased_tables => $aliased_tables,
   );
@@ -325,7 +337,9 @@ sub _parse_join_path {
   # build first member of the @join result
   my %first_join = (kind => '', name => $initial_table);
   $initial_table =~ s/\|(.+)$//  and $first_join{alias} = $1;
-  my $table = $self->table($initial_table);
+  my $table = $self->table($initial_table)
+    or croak "...->join('$initial_table', ...) : this schema has "
+           . "no table named '$initial_table'";
   $first_join{table}       = $table;
   $first_join{primary_key} = [$table->primary_key];
   $first_join{db_table}    = $table->db_from;
@@ -377,7 +391,8 @@ sub _parse_join_path {
       my $right_table = $alias || $path->{to}->db_from;
       my %condition;
       while (my ($left_col, $right_col) = each %{$path->{on}}) {
-        $condition{"$left_table.$left_col"} = \"= $right_table.$right_col";
+        # FIXME: honor SQL::Abstract's "name_sep" setting
+        $condition{"$left_table.$left_col"} = { -ident => "$right_table.$right_col" };
       }
       my $db_table = $path->{to}->db_from;
       $db_table .= "|$alias" if $alias;
